# NanoBlogger Code Formatting Plugin
#
# Copyright 2006, Niels Heirbaut <niels@heirbaut.nl>
# This program is released under the GPL v2.

# How it works:
# This plugin utilizes ViM to generate the XHTML for the code in the entry, so
# make sure that ViM is installed. Place the plugin in the directory where the
# plugins are kept that are called for each entry change. E.g.:
#
#   * plugins/entry/mod  (>=3.2)
#   * plugins/entry/body (3.1)
#   * plugins/entry      (>=2.8)
#
# Alternatively the plugin can be placed in plugins/entry/format (more on that
# later).
#
# In an entry code can be defined by placing a line starting with
# 'CODE(<your_programming_language>){' before the code and a line starting with
# '}CODE' right after the code. No white spaces are allowed before or after
# these lines. Example:
#
# CODE(c){
# #include <stdio.h>
#
# int main( int argc, char* argv[] )
# {
#     (void)printf( "Hello, World!\n" );
#
#     return 0;
# }
# }CODE
#
# The XHTML generated in the entry needs to be formatted so add the following
# to the CSS if a dark code background is preferred:
#
# .nbcode { line-height: 120%; }
# .nbcode .Comment { color: #00ffff; font-weight: bold; }
# .nbcode .Constant { color: #ff40ff; font-weight: bold; }
# .nbcode .Error { color: #ffffff; background-color: #ff6060; font-weight: bold; }
# .nbcode .Identifier { color: #00ffff; font-weight: bold; }
# .nbcode .PreProc { color: #8080ff; font-weight: bold; }
# .nbcode .Special { color: #ff6060; font-weight: bold; }
# .nbcode .Statement { color: #ffff00; font-weight: bold; }
# .nbcode .Title { color: #ff40ff; font-weight: bold; }
# .nbcode .Todo { color: #808080; background-color: #ffff00; }
# .nbcode .Type { color: #00ff00; font-weight: bold; }
# .nbcode .lnr { color: #ffff00; font-weight: bold; }
# .nbcode body { color: #ffffff; background-color: #000000; }
# .nbcode pre { color: #ffffff; background-color: #000000; }
#
# Or add the following if a light background is preferred:
#
# .nbcode { line-height: 120%; }
# .nbcode .Comment { color: #8080ff; }
# .nbcode .Constant { color: #ff6060; }
# .nbcode .Error { color: #ffffff; background-color: #ff6060; font-weight: bold; }
# .nbcode .Identifier { color: #00ffff; }
# .nbcode .PreProc { color: #ff40ff; }
# .nbcode .Special { color: #ff40ff; }
# .nbcode .Statement { color: #ffff00; }
# .nbcode .Title { color: #ff40ff; }
# .nbcode .Todo { color: #808080; background-color: #ffff00; }
# .nbcode .Type { color: #00ff00; }
# .nbcode .lnr { color: #ffff00; }
# .nbcode body { color: #000000; background-color: #ffffff; }
# .nbcode pre { color: #000000; background-color: #ffffff; }
#
# If the plugin has been placed in plugins/entry/format then the format can be
# specified on a per entry basis. Just add 'code' right after the 'FORMAT:'
# meta tag of the entry when editing.
#
# This plugin plays nice with Markdown as long as this plugin is called before
# markdown.sh. This is accomplished automatically if the plugin in NOT placed
# in plugins/entry/format because code.sh comes before markdown.sh
# alphabetically. If it IS place in plugins/entry/format be sure to have 'code'
# before 'markdown' after the 'FORMAT:' meta tag.

# The following can also be set from:
#   * Environment variables
#   * The blog.conf file
#   * The nb.conf file
#
: ${NBCODE_VIM:=vim}        # Where ViM can be found
: ${NBCODE_TABSTOP:=4}      # Number of spaces a tab counts for
: ${NBCODE_SHIFTWIDTH:=4}   # Number of spaces for each indent
: ${NBCODE_EXPANDTAB:=yes}  # Replace tabs with spaces (yes/no)
: ${NBCODE_NUMBERING:=yes}  # Add line numbering (yes/no)


# Default output filename for ViM
OUTFILE="Untitled.html"

if [ $NBCODE_EXPANDTAB != "expandtab" -a $NBCODE_EXPANDTAB != "noexpandtab" ]
then
    if [ $NBCODE_EXPANDTAB = "yes" ]; then
        NBCODE_EXPANDTAB="expandtab"
    else
        NBCODE_EXPANDTAB="noexpandtab"
    fi
fi

if [ $NBCODE_NUMBERING != "number" -a $NBCODE_NUMBERING != "nonumber" ]; then
    if [ $NBCODE_NUMBERING = "yes" ]; then
        NBCODE_NUMBERING="number"
    else
        NBCODE_NUMBERING="nonumber"
    fi
fi

set_preamble(){
    PREAMBLE=
    PREAMBLE=$( echo "$NB_EntryBody" |\
                sed -n -e '1,/^CODE(.\+){$/ { /^CODE(.\+){$/b; P }' )
    NB_EntryBody=$( echo "$NB_EntryBody" | sed -e '/^CODE(.\+){$/,$!d' )
}

set_programming_language(){
    PROGLANG=
    PROGLANG=$( echo "$NB_EntryBody" |\
                grep -m 1 '^CODE(.\+){$' |\
                sed -e 's/^CODE(\(.\+\)){/\1/' )
}

set_code(){
    CODE=
    CODE=$( echo "$NB_EntryBody" | sed -e '/^}CODE$/q' )
    CODE=$( echo "$CODE" | sed -e '/^CODE(.\+){$/d' -e '/^}CODE$/d' )
    NB_EntryBody=$( echo "$NB_EntryBody" | sed -e '/^}CODE$/,$!d' )
    
    local line=$( echo "$NB_EntryBody" | sed -n '1 p' )
    if [ "x$line" = "x}CODE" ] ; then
        NB_EntryBody=$( echo "$NB_EntryBody" | sed -e '1 d' )
    fi
}

make_html_from_code()
{
    [ -z "$PROGLANG" ] && return
    [ -z "$CODE" ] && return

    echo "$CODE" |\
    $NBCODE_VIM +"set $NBCODE_NUMBERING" \
                +"syntax on" \
                +"let html_use_css=1" \
                +"let use_xhtml=1" \
                +"set syntax=$PROGLANG $NBCODE_EXPANDTAB" \
                +"sw=$NBCODE_SHIFTWIDTH ts=$NBCODE_TABSTOP" \
                +"run! syntax/2html.vim" \
                +"w" +"qa!" - 2>&1 >/dev/null |\
    grep -v 'not.* a terminal' 2>&1 >/dev/null

    CODE=$( sed -e '/^<body>/,/^<\/body>/!d' \
                -e '/^<body>/d' \
                -e '/^<\/body>/d' $OUTFILE )
    CODE=$( echo "<div class=\"nbcode\">"; echo "$CODE"; echo "</div>" )
}

# I wish Bash had a do..while
CODE='##DUMMY##'
until [ -z "$CODE" ] ; do
    set_preamble
    set_programming_language
    set_code
    make_html_from_code

    if [ ! -z "$CODE" ] ; then
        NB_EntryBody=$( echo "$PREAMBLE"; echo "$CODE"; echo "$NB_EntryBody" )
    else
        NB_EntryBody="$PREAMBLE"
    fi
done

rm -f $OUTFILE

